"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var observable_1 = require("data/observable");
var observable_array_1 = require("data/observable-array");
var builder_1 = require("ui/builder");
var properties_1 = require("ui/core/properties");
var view_1 = require("ui/core/view");
var weak_event_listener_1 = require("ui/core/weak-event-listener");
var label_1 = require("ui/label");
var stack_layout_1 = require("ui/layouts/stack-layout");
var proxy_view_container_1 = require("ui/proxy-view-container");
__export(require("ui/core/view"));
var knownTemplates;
(function (knownTemplates) {
    knownTemplates.foregroundItemTemplate = "foregroundItemTemplate";
    knownTemplates.containerItemTemplate = "containerItemTemplate";
})(knownTemplates = exports.knownTemplates || (exports.knownTemplates = {}));
var knownMultiTemplates;
(function (knownMultiTemplates) {
    knownMultiTemplates.foregroundItemTemplates = "foregroundItemTemplates";
    knownMultiTemplates.containerItemTemplates = "containerItemTemplates";
})(knownMultiTemplates = exports.knownMultiTemplates || (exports.knownMultiTemplates = {}));
var autoEffectiveRowHeight = -1;
var defaultFoldedRowHeight = 44;
var FoldingListViewBase = (function (_super) {
    __extends(FoldingListViewBase, _super);
    function FoldingListViewBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.toggleMode = false;
        _this._effectiveFoldedRowHeight = view_1.Length.toDevicePixels(defaultFoldedRowHeight, autoEffectiveRowHeight);
        _this._defaultForegroundItemTemplate = {
            key: "default",
            createView: function () {
                if (_this.foregroundItemTemplate) {
                    return builder_1.parse(_this.foregroundItemTemplate, _this);
                }
                return undefined;
            }
        };
        _this._foregroundItemTemplatesInternal = new Array(_this._defaultForegroundItemTemplate);
        _this._defaultContainerItemTemplate = {
            key: "default",
            createView: function () {
                if (_this.containerItemTemplate) {
                    return builder_1.parse(_this.containerItemTemplate, _this);
                }
                return undefined;
            }
        };
        _this._containerItemTemplatesInternal = new Array(_this._defaultContainerItemTemplate);
        _this._cellExpanded = new Array();
        _this._itemTemplateSelectorBindable = new label_1.Label();
        _this._cachedDetailData = new Array();
        return _this;
    }
    Object.defineProperty(FoldingListViewBase.prototype, "backViewColor", {
        get: function () {
            return this.style.backViewColor;
        },
        set: function (value) {
            this.style.backViewColor = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FoldingListViewBase.prototype, "itemTemplateSelector", {
        get: function () {
            return this._itemTemplateSelector;
        },
        set: function (value) {
            var _this = this;
            if (typeof value === "string") {
                this._itemTemplateSelectorBindable.bind({
                    sourceProperty: null,
                    targetProperty: "templateKey",
                    expression: value
                });
                this._itemTemplateSelector = function (item, index, items) {
                    item["$index"] = index;
                    _this._itemTemplateSelectorBindable.bindingContext = item;
                    return _this._itemTemplateSelectorBindable.get("templateKey");
                };
            }
            else if (typeof value === "function") {
                this._itemTemplateSelector = value;
            }
        },
        enumerable: true,
        configurable: true
    });
    FoldingListViewBase.prototype.resetExpandedStates = function () {
        this._cachedDetailData = new Array();
        this._cellExpanded = new Array();
    };
    FoldingListViewBase.prototype.invalidateChachedDetailData = function (index) {
        this._setCachedDetailData(index, undefined);
    };
    FoldingListViewBase.prototype._getForegroundItemTemplate = function (index) {
        var templateKey = "default";
        if (this.itemTemplateSelector) {
            var dataItem = this._getDataItem(index);
            templateKey = this._itemTemplateSelector(dataItem, index, this.items);
        }
        for (var loop = 0, length_1 = this._foregroundItemTemplatesInternal.length; loop < length_1; loop++) {
            if (this._foregroundItemTemplatesInternal[loop].key === templateKey) {
                return this._foregroundItemTemplatesInternal[loop];
            }
        }
        return this._foregroundItemTemplatesInternal[0];
    };
    FoldingListViewBase.prototype._getContainerItemTemplate = function (index) {
        var templateKey = "default";
        if (this.itemTemplateSelector) {
            var dataItem = this._getDataItem(index);
            templateKey = this._itemTemplateSelector(dataItem, index, this.items);
        }
        for (var loop = 0, length_2 = this._containerItemTemplatesInternal.length; loop < length_2; loop++) {
            if (this._containerItemTemplatesInternal[loop].key === templateKey) {
                return this._containerItemTemplatesInternal[loop];
            }
        }
        return this._containerItemTemplatesInternal[0];
    };
    FoldingListViewBase.prototype._getDetailDataLoaderPromise = function (index) {
        if (this.detailDataLoader) {
            return this.detailDataLoader(this._getDataItem(index), index);
        }
        return null;
    };
    FoldingListViewBase.prototype._prepareItem = function (item, index) {
        if (item) {
            item.bindingContext = this._getDataItem(index);
        }
    };
    FoldingListViewBase.prototype._onFoldedRowHeightPropertyChanged = function (oldValue, newValue) {
        this.refresh();
    };
    FoldingListViewBase.prototype._onItemsChanged = function (args) {
        this.refresh();
    };
    FoldingListViewBase.prototype._getCachedDetailData = function (index) {
        return this._cachedDetailData[index];
    };
    FoldingListViewBase.prototype._setCachedDetailData = function (index, value) {
        this._cachedDetailData[index] = value;
    };
    FoldingListViewBase.prototype._getIsCellExpandedIn = function (index) {
        return this._cellExpanded[index];
    };
    FoldingListViewBase.prototype._setIsCellExpandedIn = function (index, value) {
        this._cellExpanded[index] = value;
    };
    FoldingListViewBase.prototype._getDataItem = function (index) {
        var thisItems = this.items;
        return thisItems.getItem ? thisItems.getItem(index) : thisItems[index];
    };
    FoldingListViewBase.prototype._checkAndWrapProxyContainers = function (view) {
        if (view instanceof proxy_view_container_1.ProxyViewContainer) {
            var sp = new stack_layout_1.StackLayout();
            sp.addChild(view);
            return sp;
        }
        return view;
    };
    FoldingListViewBase.prototype.updateEffectiveFoldedRowHeight = function () {
        exports.foldedRowHeightProperty.coerce(this);
    };
    FoldingListViewBase.itemLoadingEvent = "itemLoading";
    FoldingListViewBase.loadMoreItemsEvent = "loadMoreItems";
    FoldingListViewBase.knownFunctions = ["itemTemplateSelector", "detailDataLoader"];
    FoldingListViewBase = __decorate([
        view_1.CSSType("FoldingListView")
    ], FoldingListViewBase);
    return FoldingListViewBase;
}(view_1.View));
exports.FoldingListViewBase = FoldingListViewBase;
exports.itemsProperty = new properties_1.Property({
    name: "items",
    valueChanged: function (target, oldValue, newValue) {
        if (oldValue instanceof observable_1.Observable) {
            weak_event_listener_1.removeWeakEventListener(oldValue, observable_array_1.ObservableArray.changeEvent, target._onItemsChanged, target);
        }
        if (newValue instanceof observable_1.Observable) {
            weak_event_listener_1.addWeakEventListener(newValue, observable_array_1.ObservableArray.changeEvent, target._onItemsChanged, target);
        }
        target.refresh();
    }
});
exports.itemsProperty.register(FoldingListViewBase);
exports.foregroundItemTemplateProperty = new properties_1.Property({
    name: "foregroundItemTemplate",
    valueChanged: function (target) {
        target.refresh();
    },
});
exports.foregroundItemTemplateProperty.register(FoldingListViewBase);
exports.containerItemTemplateProperty = new properties_1.Property({
    name: "containerItemTemplate",
    valueChanged: function (target) {
        target.refresh();
    },
});
exports.containerItemTemplateProperty.register(FoldingListViewBase);
exports.foldsCountProperty = new properties_1.CoercibleProperty({
    name: "foldsCount",
    defaultValue: 3,
    coerceValue: function (target, value) {
        return value <= 2 ? 3 : value;
    },
    valueChanged: function (target, oldValue, newValue) {
        target.refresh();
    },
    valueConverter: function (v) { return +v; },
});
exports.foldsCountProperty.register(FoldingListViewBase);
exports.foldedRowHeightProperty = new properties_1.CoercibleProperty({
    name: "foldedRowHeight",
    defaultValue: defaultFoldedRowHeight,
    equalityComparer: view_1.Length.equals,
    coerceValue: function (target, value) {
        return target.nativeViewProtected && value > 0 ? value : defaultFoldedRowHeight;
    },
    valueChanged: function (target, oldValue, newValue) {
        target._effectiveFoldedRowHeight = view_1.Length.toDevicePixels(newValue, autoEffectiveRowHeight);
        target._onFoldedRowHeightPropertyChanged(oldValue, newValue);
    },
    valueConverter: view_1.Length.parse
});
exports.foldedRowHeightProperty.register(FoldingListViewBase);
exports.backViewColorProperty = new view_1.CssProperty({
    name: "backViewColor",
    defaultValue: new view_1.Color("white"),
    cssName: "back-view-color",
    equalityComparer: view_1.Color.equals,
    valueConverter: function (v) { return new view_1.Color(v); },
});
exports.backViewColorProperty.register(view_1.Style);
exports.foregroundItemTemplatesProperty = new properties_1.Property({
    name: "foregroundItemTemplates",
    valueConverter: function (value) {
        if (typeof value === "string") {
            return builder_1.parseMultipleTemplates(value);
        }
        return value;
    }
});
exports.foregroundItemTemplatesProperty.register(FoldingListViewBase);
exports.containerItemTemplatesProperty = new properties_1.Property({
    name: "containerItemTemplates",
    valueConverter: function (value) {
        if (typeof value === "string") {
            return builder_1.parseMultipleTemplates(value);
        }
        return value;
    }
});
exports.containerItemTemplatesProperty.register(FoldingListViewBase);
exports.foldAnimationDurationProperty = new properties_1.Property({
    name: "foldAnimationDuration",
    defaultValue: 330,
    valueConverter: function (v) { return +v; },
    valueChanged: function (target, oldValue, newValue) {
        target.refresh();
    },
});
exports.foldAnimationDurationProperty.register(FoldingListViewBase);
exports.toggleModeProperty = new properties_1.Property({
    name: "toggleMode",
    defaultValue: false,
    valueConverter: view_1.booleanConverter,
});
exports.toggleModeProperty.register(FoldingListViewBase);
