"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var observable_1 = require("data/observable");
var view_1 = require("ui/core/view");
var utils = require("utils/utils");
var folding_list_view_common_1 = require("./folding-list-view-common");
__export(require("./folding-list-view-common"));
var infinity = view_1.layout.makeMeasureSpec(0, view_1.layout.UNSPECIFIED);
var FoldingListView = (function (_super) {
    __extends(FoldingListView, _super);
    function FoldingListView() {
        var _this = _super.call(this) || this;
        _this.widthMeasureSpec = 0;
        _this.nativeViewProtected = _this._ios = UITableView.new();
        _this._ios.registerClassForCellReuseIdentifier(FoldingListViewCell.class(), _this._defaultForegroundItemTemplate.key);
        _this._ios.separatorColor = utils.ios.getter(UIColor, UIColor.clearColor);
        _this._ios.rowHeight = UITableViewAutomaticDimension;
        _this._ios.dataSource = _this._dataSource = FoldingListViewDataSource.initWithOwner(new WeakRef(_this));
        _this._delegate = FoldingListViewDelegate.initWithOwner(new WeakRef(_this));
        _this._heights = new Array();
        _this._map = new Map();
        _this._setNativeClipToBounds();
        return _this;
    }
    FoldingListView.prototype._setNativeClipToBounds = function () {
        this._ios.clipsToBounds = true;
    };
    FoldingListView.prototype.onLoaded = function () {
        _super.prototype.onLoaded.call(this);
        if (this._isDataDirty) {
            this.refresh();
        }
        this._ios.delegate = this._delegate;
    };
    FoldingListView.prototype.onUnloaded = function () {
        this._ios.delegate = null;
        _super.prototype.onUnloaded.call(this);
    };
    Object.defineProperty(FoldingListView.prototype, "ios", {
        get: function () {
            return this._ios;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FoldingListView.prototype, "_childrenCount", {
        get: function () {
            return this._map.size;
        },
        enumerable: true,
        configurable: true
    });
    FoldingListView.prototype.eachChildView = function (callback) {
        this._map.forEach(function (view, key) {
            callback(view.foreground);
            callback(view.container);
        });
    };
    FoldingListView.prototype.scrollToIndex = function (index, animated) {
        if (animated === void 0) { animated = true; }
        if (!this._ios) {
            return;
        }
        var itemsLength = this.items ? this.items.length : 0;
        if (itemsLength > 0) {
            if (index < 0) {
                index = 0;
            }
            else if (index >= itemsLength) {
                index = itemsLength - 1;
            }
            this._ios.scrollToRowAtIndexPathAtScrollPositionAnimated(NSIndexPath.indexPathForItemInSection(index, 0), 1, animated);
        }
    };
    FoldingListView.prototype.refresh = function () {
        this._map.forEach(function (view, nativeView) {
            if (!(view.foreground.bindingContext instanceof observable_1.Observable)) {
                view.foreground.bindingContext = null;
            }
            if (!(view.container.bindingContext instanceof observable_1.Observable)) {
                view.foreground.bindingContext = null;
            }
        });
        if (this.isLoaded) {
            this._ios.reloadData();
            this.requestLayout();
            this._isDataDirty = false;
        }
        else {
            this._isDataDirty = true;
        }
    };
    FoldingListView.prototype.getHeight = function (index) {
        return this._heights[index];
    };
    FoldingListView.prototype.setHeight = function (index, value) {
        this._heights[index] = value;
    };
    FoldingListView.prototype._onFoldedRowHeightPropertyChanged = function (oldValue, newValue) {
        var value = view_1.layout.toDeviceIndependentPixels(this._effectiveFoldedRowHeight);
        if (value > 0) {
            this._ios.estimatedRowHeight = value;
        }
        _super.prototype._onFoldedRowHeightPropertyChanged.call(this, oldValue, newValue);
    };
    FoldingListView.prototype.requestLayout = function () {
        if (!this._preparingCell) {
            _super.prototype.requestLayout.call(this);
        }
    };
    FoldingListView.prototype.measure = function (widthMeasureSpec, heightMeasureSpec) {
        this.widthMeasureSpec = widthMeasureSpec;
        var changed = this._setCurrentMeasureSpecs(widthMeasureSpec, heightMeasureSpec);
        _super.prototype.measure.call(this, widthMeasureSpec, heightMeasureSpec);
        if (changed) {
            this._ios.reloadData();
        }
    };
    FoldingListView.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        var _this = this;
        _super.prototype.onMeasure.call(this, widthMeasureSpec, heightMeasureSpec);
        this._map.forEach(function (cellView, listViewCell) {
            view_1.View.measureChild(_this, cellView.foreground, cellView.foreground._currentWidthMeasureSpec, cellView.foreground._currentHeightMeasureSpec);
            view_1.View.measureChild(_this, cellView.container, cellView.container._currentWidthMeasureSpec, cellView.container._currentHeightMeasureSpec);
        });
    };
    FoldingListView.prototype.onLayout = function (left, top, right, bottom) {
        var _this = this;
        _super.prototype.onLayout.call(this, left, top, right, bottom);
        this._map.forEach(function (cellView, listViewCell) {
            var width = view_1.layout.getMeasureSpecSize(_this.widthMeasureSpec);
            var cellHeight = _this.getHeight(cellView.index);
            if (cellHeight) {
                _this._layoutConstraintedView(cellView.foreground, width, cellHeight.foreground);
                _this._layoutConstraintedView(cellView.container, width, cellHeight.container);
            }
        });
    };
    FoldingListView.prototype._prepareCell = function (cell, indexPath) {
        var _this = this;
        var cellHeight;
        var isForegroundViewToBeConstrainedIn = false;
        var isContainerViewToBeConstrainedIn = false;
        var index = indexPath.row;
        try {
            this._preparingCell = true;
            var foregroundView = cell.foregroundViewTNS;
            if (!foregroundView) {
                foregroundView = this._getForegroundItemTemplate(index).createView();
            }
            var containerView_1 = cell.containerViewTNS;
            if (!containerView_1) {
                containerView_1 = this._getContainerItemTemplate(index).createView();
            }
            this.notify({
                eventName: FoldingListView.itemLoadingEvent,
                object: this,
                index: index,
                view: {
                    foreground: foregroundView,
                    container: containerView_1,
                },
                ios: cell,
                android: undefined,
            });
            foregroundView = this._checkAndWrapProxyContainers(foregroundView);
            containerView_1 = this._checkAndWrapProxyContainers(containerView_1);
            if (!cell.foregroundViewTNS) {
                cell.foregroundViewWeakRef = new WeakRef(foregroundView);
                isForegroundViewToBeConstrainedIn = true;
            }
            else if (cell.foregroundViewTNS !== foregroundView) {
                isForegroundViewToBeConstrainedIn = true;
                this._removeContainer(cell);
                cell.foregroundViewTNS.nativeViewProtected.removeFromSuperview();
                cell.foregroundViewWeakRef = new WeakRef(foregroundView);
            }
            this._prepareItem(foregroundView, index);
            if (!cell.containerViewTNS) {
                isContainerViewToBeConstrainedIn = true;
                cell.containerViewWeakRef = new WeakRef(containerView_1);
            }
            else if (cell.containerViewTNS !== containerView_1) {
                isContainerViewToBeConstrainedIn = true;
                this._removeContainer(cell);
                cell.containerViewTNS.nativeViewProtected.removeFromSuperview();
                cell.containerViewWeakRef = new WeakRef(containerView_1);
            }
            if (!this.detailDataLoader) {
                this._prepareItem(containerView_1, index);
            }
            else {
                var cachedData = this._getCachedDetailData(index);
                if (cachedData) {
                    cell._bindContainerView(index, cachedData);
                }
                else if (this._getIsCellExpandedIn(index)) {
                    this._getDetailDataLoaderPromise(index)
                        .then(function (result) {
                        _this._setCachedDetailData(index, result);
                        containerView_1.bindingContext = result;
                    })
                        .catch(function (e) { console.error("ERROR LOADING DETAILS:", e); });
                }
            }
            var cellView = {
                foreground: foregroundView,
                container: containerView_1,
                index: index,
            };
            this._map.set(cell, cellView);
            if (foregroundView && !foregroundView.parent) {
                this._addView(foregroundView);
            }
            if (containerView_1 && !containerView_1.parent) {
                this._addView(containerView_1);
            }
            containerView_1.marginTop = foregroundView.marginTop;
            if (isForegroundViewToBeConstrainedIn) {
                this._prepareConstrainedView(foregroundView);
            }
            if (isContainerViewToBeConstrainedIn) {
                this._prepareConstrainedView(containerView_1);
            }
            cellHeight = this._layoutCell(cellView);
            var estimatedRowHeight = view_1.layout.toDeviceIndependentPixels(cellHeight.foreground);
            if (this._ios.estimatedRowHeight !== estimatedRowHeight) {
                this._ios.estimatedRowHeight = estimatedRowHeight;
            }
            cell.resetNativeViews(cellHeight);
        }
        finally {
            this._preparingCell = false;
        }
        return cellHeight;
    };
    FoldingListView.prototype._removeContainer = function (cell) {
        var foregroundView = cell.foregroundViewTNS;
        var containerView = cell.containerViewTNS;
        if (!(foregroundView.parent instanceof FoldingListView)) {
            this._removeView(foregroundView.parent);
        }
        if (!(containerView.parent instanceof FoldingListView)) {
            this._removeView(containerView.parent);
        }
        var preparing = this._preparingCell;
        this._preparingCell = true;
        foregroundView.parent._removeView(foregroundView);
        containerView.parent._removeView(containerView);
        this._preparingCell = preparing;
        this._map.delete(cell);
    };
    FoldingListView.prototype.isItemAtIndexVisible = function (index) {
        var indexes = Array.from(this._ios.indexPathsForVisibleRows);
        return indexes.some(function (visIndex) { return visIndex.row === index; });
    };
    FoldingListView.prototype[folding_list_view_common_1.backViewColorProperty.getDefault] = function () {
        return (FoldingListViewCell.alloc().init()).backViewColor;
    };
    FoldingListView.prototype[folding_list_view_common_1.backViewColorProperty.setNative] = function (value) {
        var actualColor = value instanceof view_1.Color ? value.ios : value;
        this._map.forEach(function (view, cell) { cell.backViewColor = actualColor; });
    };
    FoldingListView.prototype[folding_list_view_common_1.foregroundItemTemplatesProperty.getDefault] = function () {
        return null;
    };
    FoldingListView.prototype[folding_list_view_common_1.foregroundItemTemplatesProperty.setNative] = function (value) {
        this._foregroundItemTemplatesInternal = new Array(this._defaultForegroundItemTemplate);
        if (value) {
            for (var _i = 0, value_1 = value; _i < value_1.length; _i++) {
                var item = value_1[_i];
                this._ios.registerClassForCellReuseIdentifier(FoldingListViewCell.class(), item.key);
            }
            this._foregroundItemTemplatesInternal = this._foregroundItemTemplatesInternal.concat(value);
        }
        this.refresh();
    };
    FoldingListView.prototype[folding_list_view_common_1.containerItemTemplatesProperty.getDefault] = function () {
        return null;
    };
    FoldingListView.prototype[folding_list_view_common_1.containerItemTemplatesProperty.setNative] = function (value) {
        this._containerItemTemplatesInternal = new Array(this._defaultContainerItemTemplate);
        if (value) {
            for (var _i = 0, value_2 = value; _i < value_2.length; _i++) {
                var item = value_2[_i];
                this._ios.registerClassForCellReuseIdentifier(FoldingListViewCell.class(), item.key);
            }
            this._containerItemTemplatesInternal = this._containerItemTemplatesInternal.concat(value);
        }
        this.refresh();
    };
    FoldingListView.prototype._measureConstraintedChild = function (view, measuredHeight) {
        return view_1.View.measureChild(this, view, this.widthMeasureSpec - (view._constraintLeft + view._constraintRight), measuredHeight);
    };
    FoldingListView.prototype._layoutConstraintedView = function (view, width, height) {
        view_1.View.layoutChild(this, view, 0, 0, width - (view._constraintLeft + view._constraintRight), height - view._constraintTop);
    };
    FoldingListView.prototype._layoutCell = function (cellView) {
        if (cellView) {
            var measureForegroundSize = this._measureConstraintedChild(cellView.foreground, view_1.layout.makeMeasureSpec(this._effectiveFoldedRowHeight, view_1.layout.EXACTLY));
            var measuredContainerSize = this._measureConstraintedChild(cellView.container, infinity);
            var height = {
                foreground: measureForegroundSize.measuredHeight + cellView.foreground._constraintTop,
                container: measuredContainerSize.measuredHeight + cellView.container._constraintTop,
            };
            this.setHeight(cellView.index, height);
            return height;
        }
        return {
            foreground: this._effectiveFoldedRowHeight,
            container: this._effectiveFoldedRowHeight,
        };
    };
    FoldingListView.prototype._prepareConstrainedView = function (view) {
        view._constraintTop = view_1.PercentLength.toDevicePixels(view.marginTop);
        view._constraintLeft = view_1.PercentLength.toDevicePixels(view.marginLeft) + this.effectivePaddingLeft;
        view._constraintBottom = view_1.PercentLength.toDevicePixels(view.marginBottom);
        view._constraintRight = view_1.PercentLength.toDevicePixels(view.marginRight) + this.effectivePaddingRight;
        view.margin = "0";
    };
    return FoldingListView;
}(folding_list_view_common_1.FoldingListViewBase));
exports.FoldingListView = FoldingListView;
var FoldingListViewCell = (function (_super) {
    __extends(FoldingListViewCell, _super);
    function FoldingListViewCell() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(FoldingListViewCell.prototype, "foregroundViewTNS", {
        get: function () {
            return this.foregroundViewWeakRef ? this.foregroundViewWeakRef.get() : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FoldingListViewCell.prototype, "containerViewTNS", {
        get: function () {
            return this.containerViewWeakRef ? this.containerViewWeakRef.get() : null;
        },
        enumerable: true,
        configurable: true
    });
    FoldingListViewCell.prototype.willMoveToSuperview = function (newSuperview) {
        var parent = (this.foregroundViewTNS ? this.foregroundViewTNS.parent : null);
        if (parent && !newSuperview) {
            parent._removeContainer(this);
        }
    };
    FoldingListViewCell.prototype.resetNativeViews = function (cellHeight) {
        var parent = (this.foregroundViewTNS ? this.foregroundViewTNS.parent : null);
        for (var loop = this.contentView.subviews.count - 1; loop >= 0; loop--) {
            this.contentView.subviews.objectAtIndex(loop).removeFromSuperview();
        }
        this._initForegroundView(cellHeight.foreground);
        this._initContainerView(cellHeight.container);
        this.backViewColor = parent.backViewColor.ios;
        this.commonInit();
    };
    FoldingListViewCell.prototype.resetContainerViewHeightContraint = function (newHeight) {
        var topConstraintValue = view_1.layout.toDeviceIndependentPixels(this.containerViewTNS._constraintTop);
        if (this._containerViewHeightConstraints) {
            NSLayoutConstraint.deactivateConstraints(this._containerViewHeightConstraints);
        }
        this._containerViewHeightConstraints = NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("V:[layer(==" + (view_1.layout.toDeviceIndependentPixels(newHeight) - topConstraintValue) + ")]", 0, null, { layer: this.containerView });
        NSLayoutConstraint.activateConstraints(this._containerViewHeightConstraints);
    };
    FoldingListViewCell.prototype.animationDurationType = function (itemIndex, type) {
        var parent = (this.foregroundViewTNS ? this.foregroundViewTNS.parent : null);
        return parent.foldAnimationDuration / 1000;
    };
    FoldingListViewCell.prototype._bindContainerView = function (index, dataItem) {
        var containerView = this.containerViewTNS;
        var parent = containerView.parent;
        var width = view_1.layout.getMeasureSpecSize(parent.widthMeasureSpec);
        containerView.bindingContext = dataItem;
        var size = parent._measureConstraintedChild(containerView, infinity);
        var cellHeight = parent.getHeight(index);
        cellHeight.container = size.measuredHeight + containerView._constraintTop;
        parent.setHeight(index, cellHeight);
        if (containerView && containerView.isLayoutRequired) {
            parent._layoutConstraintedView(containerView, width, cellHeight.container);
        }
        this.resetContainerViewHeightContraint(cellHeight.container);
    };
    FoldingListViewCell.prototype._initForegroundView = function (height) {
        var topConstraintValue = view_1.layout.toDeviceIndependentPixels(this.foregroundViewTNS._constraintTop);
        var foregroundView = RotatedView.alloc().initWithFrame(CGRectZero);
        foregroundView.translatesAutoresizingMaskIntoConstraints = false;
        foregroundView.addSubview(this.foregroundViewTNS.nativeViewProtected);
        this.contentView.addSubview(foregroundView);
        NSLayoutConstraint.activateConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("V:[layer(==" + (view_1.layout.toDeviceIndependentPixels(height) - topConstraintValue) + ")]", 0, null, { layer: foregroundView }));
        NSLayoutConstraint.activateConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("H:|-" + view_1.layout.toDeviceIndependentPixels(this.foregroundViewTNS._constraintLeft) + "-[layer]-" + view_1.layout.toDeviceIndependentPixels(this.foregroundViewTNS._constraintRight) + "-|", 0, null, { layer: foregroundView }));
        var top = NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("V:|-" + topConstraintValue + "-[layer]", 0, null, { layer: foregroundView });
        NSLayoutConstraint.activateConstraints(top);
        this.foregroundView = foregroundView;
        this.foregroundViewTop = top.objectAtIndex(0);
    };
    FoldingListViewCell.prototype._initContainerView = function (height) {
        var topConstraintValue = view_1.layout.toDeviceIndependentPixels(this.containerViewTNS._constraintTop);
        var containerView = UIView.alloc().initWithFrame(CGRectZero);
        containerView.translatesAutoresizingMaskIntoConstraints = false;
        containerView.addSubview(this.containerViewTNS.nativeViewProtected);
        this.contentView.addSubview(containerView);
        this.containerView = containerView;
        this.resetContainerViewHeightContraint(height);
        NSLayoutConstraint.activateConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("H:|-" + view_1.layout.toDeviceIndependentPixels(this.containerViewTNS._constraintLeft) + "-[layer]-" + view_1.layout.toDeviceIndependentPixels(this.containerViewTNS._constraintRight) + "-|", 0, null, { layer: containerView }));
        var top = NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("V:|-" + topConstraintValue + "-[layer]", 0, null, { layer: containerView });
        NSLayoutConstraint.activateConstraints(top);
        this.containerViewTop = top.objectAtIndex(0);
        containerView.layoutIfNeeded();
    };
    return FoldingListViewCell;
}(FoldingCell));
var FoldingListViewDelegate = (function (_super) {
    __extends(FoldingListViewDelegate, _super);
    function FoldingListViewDelegate() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    FoldingListViewDelegate_1 = FoldingListViewDelegate;
    FoldingListViewDelegate.initWithOwner = function (owner) {
        var delegate = FoldingListViewDelegate_1.new();
        delegate._owner = owner;
        return delegate;
    };
    FoldingListViewDelegate.prototype.tableViewHeightForRowAtIndexPath = function (tableView, indexPath) {
        var owner = this._owner.get();
        var cellHeight = owner.getHeight(indexPath.row);
        if (!cellHeight) {
            return view_1.layout.toDeviceIndependentPixels(owner._effectiveFoldedRowHeight);
        }
        return view_1.layout.toDeviceIndependentPixels(owner._getIsCellExpandedIn(indexPath.row) ? cellHeight.container : cellHeight.foreground);
    };
    FoldingListViewDelegate.prototype.tableViewDidSelectRowAtIndexPath = function (tableView, indexPath) {
        var _this = this;
        var cell = tableView.cellForRowAtIndexPath(indexPath);
        var owner = this._owner.get();
        if (cell.isAnimating()) {
            return;
        }
        var isExpandedIn = !owner._getIsCellExpandedIn(indexPath.row);
        var index = indexPath.row;
        if (isExpandedIn && owner.detailDataLoader) {
            owner._getDetailDataLoaderPromise(index)
                .then(function (value) {
                cell._bindContainerView(index, value);
                owner._setCachedDetailData(index, value);
                setTimeout(function () { _this._performCellUnfold(cell, index, isExpandedIn); }, 1);
            })
                .catch(function (e) { console.error("ERROR LOADING DETAILS:", e); });
        }
        else {
            this._performCellUnfold(cell, index, isExpandedIn);
        }
        if (!isExpandedIn) {
            owner.invalidateChachedDetailData(index);
        }
    };
    FoldingListViewDelegate.prototype.tableViewWillDisplayCellForRowAtIndexPath = function (tableView, cell, indexPath) {
        var foldingCell = cell;
        var owner = this._owner.get();
        var isExpandedIn = owner._getIsCellExpandedIn(indexPath.row);
        if (owner && (indexPath.row === owner.items.length - 1)) {
            owner.notify({
                eventName: folding_list_view_common_1.FoldingListViewBase.loadMoreItemsEvent,
                object: owner,
            });
        }
        foldingCell.unfoldAnimatedCompletion(isExpandedIn, false, null);
    };
    FoldingListViewDelegate.prototype._performCellUnfold = function (cell, index, isExpandedIn) {
        var owner = this._owner.get();
        if (owner.toggleMode && isExpandedIn) {
            var expandedIndex_1 = owner._cellExpanded.findIndex(function (value) { return value; });
            var expandedCell_1;
            owner._map.forEach(function (value, key) {
                if (value.index === expandedIndex_1) {
                    expandedCell_1 = key;
                }
            });
            if (!expandedCell_1) {
                owner._setIsCellExpandedIn(expandedIndex_1, false);
            }
            else {
                this._performCellUnfold(expandedCell_1, expandedIndex_1, false);
            }
        }
        owner._setIsCellExpandedIn(index, isExpandedIn);
        cell.unfoldAnimatedCompletion(isExpandedIn, true, null);
        var duration = owner.foldsCount * (owner.foldAnimationDuration / 1000);
        if (isExpandedIn) {
            duration /= 2;
        }
        UIView.animateWithDurationDelayOptionsAnimationsCompletion(duration, 0, 131072, function () {
            owner.ios.beginUpdates();
            owner.ios.endUpdates();
        }, null);
    };
    FoldingListViewDelegate = FoldingListViewDelegate_1 = __decorate([
        ObjCClass(UITableViewDelegate)
    ], FoldingListViewDelegate);
    return FoldingListViewDelegate;
    var FoldingListViewDelegate_1;
}(NSObject));
var FoldingListViewDataSource = (function (_super) {
    __extends(FoldingListViewDataSource, _super);
    function FoldingListViewDataSource() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    FoldingListViewDataSource_1 = FoldingListViewDataSource;
    FoldingListViewDataSource.initWithOwner = function (owner) {
        var dataSource = FoldingListViewDataSource_1.new();
        dataSource._owner = owner;
        return dataSource;
    };
    FoldingListViewDataSource.prototype.tableViewNumberOfRowsInSection = function (tableView, section) {
        var owner = this._owner.get();
        return (owner && owner.items) ? owner.items.length : 0;
    };
    FoldingListViewDataSource.prototype.tableViewCellForRowAtIndexPath = function (tableView, indexPath) {
        var owner = this._owner.get();
        var cell;
        if (owner) {
            var template = owner._getForegroundItemTemplate(indexPath.row);
            cell = (tableView.dequeueReusableCellWithIdentifier(template.key) || FoldingListViewCell.new());
            var cellHeight = owner._prepareCell(cell, indexPath);
            var width = view_1.layout.getMeasureSpecSize(owner.widthMeasureSpec);
            var foregroundView = cell.foregroundViewTNS;
            if (foregroundView && foregroundView.isLayoutRequired) {
                owner._layoutConstraintedView(foregroundView, width, cellHeight.foreground);
            }
            var containerView = cell.containerViewTNS;
            if (containerView && containerView.isLayoutRequired) {
                owner._layoutConstraintedView(containerView, width, cellHeight.container);
            }
            cell.itemCount = owner.foldsCount;
        }
        else {
            cell = FoldingListViewCell.new();
        }
        return cell;
    };
    FoldingListViewDataSource = FoldingListViewDataSource_1 = __decorate([
        ObjCClass(UITableViewDataSource)
    ], FoldingListViewDataSource);
    return FoldingListViewDataSource;
    var FoldingListViewDataSource_1;
}(NSObject));
